@@ -1,6 +1,6 @@
 /*
  * Copyright © 2009 CNRS
- * Copyright © 2009-2022 Inria.  All rights reserved.
+ * Copyright © 2009-2023 Inria.  All rights reserved.
  * Copyright © 2009-2013 Université Bordeaux
  * Copyright © 2009-2011 Cisco Systems, Inc.  All rights reserved.
  * See COPYING in top-level directory.
@@ -50,7 +50,7 @@ hwloc__darwin_cpukinds_add(struct hwloc_darwin_cpukinds *kinds,
                            char cluster_type, const char *compatible)
 {
   if (kinds->nr == MAX_KINDS) {
-    if (!hwloc_hide_errors())
+    if (HWLOC_SHOW_ALL_ERRORS())
       fprintf(stderr, "hwloc/darwin: failed to add new cpukinds, already %u used\n", kinds->nr);
     return NULL;
   }
@@ -129,7 +129,7 @@ static int hwloc__darwin_look_iokit_cpukinds(struct hwloc_darwin_cpukinds *kinds
 
   kret = IORegistryEntryGetChildIterator(cpus_root, DT_PLANE, &cpus_iter);
   if (kret != KERN_SUCCESS) {
-    if (!hwloc_hide_errors())
+    if (HWLOC_SHOW_ALL_ERRORS())
       fprintf(stderr, "hwloc/darwin/cpukinds: failed to create iterator\n");
     IOObjectRelease(cpus_root);
     return -1;
@@ -162,7 +162,7 @@ static int hwloc__darwin_look_iokit_cpukinds(struct hwloc_darwin_cpukinds *kinds
       continue;
     }
     if (CFGetTypeID(ref) != CFNumberGetTypeID()) {
-      if (!hwloc_hide_errors())
+      if (HWLOC_SHOW_ALL_ERRORS())
         fprintf(stderr, "hwloc/darwin/cpukinds: unexpected `logical-cpu-id' CF type %s\n",
                 CFStringGetCStringPtr(CFCopyTypeIDDescription(CFGetTypeID(ref)), kCFStringEncodingUTF8));
       CFRelease(ref);
@@ -171,7 +171,7 @@ static int hwloc__darwin_look_iokit_cpukinds(struct hwloc_darwin_cpukinds *kinds
     {
       long long lld_value;
       if (!CFNumberGetValue(ref, kCFNumberLongLongType, &lld_value)) {
-        if (!hwloc_hide_errors())
+        if (HWLOC_SHOW_ALL_ERRORS())
           fprintf(stderr, "hwloc/darwin/cpukinds: failed to get logical-cpu-id\n");
         CFRelease(ref);
         continue;
@@ -209,19 +209,19 @@ static int hwloc__darwin_look_iokit_cpukinds(struct hwloc_darwin_cpukinds *kinds
     /* get cluster-type */
     ref = IORegistryEntrySearchCFProperty(cpus_child, DT_PLANE, CFSTR("cluster-type"), kCFAllocatorDefault, kNilOptions);
     if (!ref) {
-      if (!hwloc_hide_errors())
+      if (HWLOC_SHOW_ALL_ERRORS())
         fprintf(stderr, "hwloc/darwin/cpukinds: failed to find cluster-type\n");
       continue;
     }
     if (CFGetTypeID(ref) != CFDataGetTypeID()) {
-      if (!hwloc_hide_errors())
+      if (HWLOC_SHOW_ALL_ERRORS())
         fprintf(stderr, "hwloc/darwin/cpukinds: unexpected `cluster-type' CF type %s\n",
                 CFStringGetCStringPtr(CFCopyTypeIDDescription(CFGetTypeID(ref)), kCFStringEncodingUTF8));
       CFRelease(ref);
       continue;
     }
     if (CFDataGetLength(ref) < 2) {
-      if (!hwloc_hide_errors())
+      if (HWLOC_SHOW_ALL_ERRORS())
         fprintf(stderr, "hwloc/darwin/cpukinds: only got %ld bytes from cluster-type data\n",
                 CFDataGetLength(ref));
       CFRelease(ref);
@@ -234,7 +234,7 @@ static int hwloc__darwin_look_iokit_cpukinds(struct hwloc_darwin_cpukinds *kinds
         hwloc_debug("got cluster-type %c\n", u8_values[0]);
         cluster_type = u8_values[0];
       } else {
-        if (!hwloc_hide_errors())
+        if (HWLOC_SHOW_ALL_ERRORS())
           fprintf(stderr, "hwloc/darwin/cpukinds: got more than one character in cluster-type data %c%c...\n",
                   u8_values[0], u8_values[1]);
         CFRelease(ref);
@@ -246,12 +246,12 @@ static int hwloc__darwin_look_iokit_cpukinds(struct hwloc_darwin_cpukinds *kinds
     /* get compatible */
     ref = IORegistryEntrySearchCFProperty(cpus_child, DT_PLANE, CFSTR("compatible"), kCFAllocatorDefault, kNilOptions);
     if (!ref) {
-      if (!hwloc_hide_errors())
+      if (HWLOC_SHOW_ALL_ERRORS())
         fprintf(stderr, "hwloc/darwin/cpukinds: failed to find compatible\n");
       continue;
     }
     if (CFGetTypeID(ref) != CFDataGetTypeID()) {
-      if (!hwloc_hide_errors())
+      if (HWLOC_SHOW_ALL_ERRORS())
         fprintf(stderr, "hwloc/darwin/cpukinds: unexpected `compatible' CF type %s\n",
                 CFStringGetCStringPtr(CFCopyTypeIDDescription(CFGetTypeID(ref)), kCFStringEncodingUTF8));
       CFRelease(ref);
@@ -269,7 +269,7 @@ static int hwloc__darwin_look_iokit_cpukinds(struct hwloc_darwin_cpukinds *kinds
         if (!compatible[i] && compatible[i+1])
           compatible[i] = ';';
       if (!compatible[0]) {
-        if (!hwloc_hide_errors())
+        if (HWLOC_SHOW_ALL_ERRORS())
           fprintf(stderr, "hwloc/darwin/cpukinds: compatible is empty\n");
         CFRelease(ref);
         continue;
@@ -301,7 +301,7 @@ static int hwloc__darwin_look_iokit_cpukinds(struct hwloc_darwin_cpukinds *kinds
       kinds->kinds[i].perflevel = 0;
     } else {
       *matched_perflevels = 0;
-      if (!hwloc_hide_errors())
+      if (HWLOC_SHOW_ALL_ERRORS())
         fprintf(stderr, "hwloc/darwin/cpukinds: unrecognized cluster type %c compatible %s, cannot match perflevels\n",
                 kinds->kinds[i].cluster_type, kinds->kinds[i].compatible);
     }
@@ -456,7 +456,8 @@ static void hwloc__darwin_look_perflevel_caches(struct hwloc_topology *topology,
   int64_t size;
 
   snprintf(name, sizeof(name), "hw.perflevel%u.l1icachesize", level);
-  if (!hwloc_get_sysctlbyname(name, &size)) {
+  if (hwloc_filter_check_keep_object_type(topology, HWLOC_OBJ_L1ICACHE)
+      && !hwloc_get_sysctlbyname(name, &size)) {
     /* hw.perflevel%u.cpusperl1i missing, assume it's per PU */
     hwloc_debug("found perflevel %u l1icachesize %ld, assuming width 1\n", level, (long) size);
     hwloc__darwin_build_perflevel_cache_level(topology, cpuset, 1, HWLOC_OBJ_L1ICACHE, 1, size, linesize);
@@ -464,7 +465,8 @@ static void hwloc__darwin_look_perflevel_caches(struct hwloc_topology *topology,
   }
 
   snprintf(name, sizeof(name), "hw.perflevel%u.l1dcachesize", level);
-  if (!hwloc_get_sysctlbyname(name, &size)) {
+  if (hwloc_filter_check_keep_object_type(topology, HWLOC_OBJ_L1CACHE)
+      && !hwloc_get_sysctlbyname(name, &size)) {
     /* hw.perflevel%u.cpusperl1d missing, assume it's per PU */
     hwloc_debug("found perflevel %u l1dcachesize %ld, assuming width 1\n", level, (long) size);
     hwloc__darwin_build_perflevel_cache_level(topology, cpuset, 1, HWLOC_OBJ_L1CACHE, 1, size, linesize);
@@ -472,7 +474,8 @@ static void hwloc__darwin_look_perflevel_caches(struct hwloc_topology *topology,
   }
 
   snprintf(name, sizeof(name), "hw.perflevel%u.l2cachesize", level);
-  if (!hwloc_get_sysctlbyname(name, &size)) {
+  if (hwloc_filter_check_keep_object_type(topology, HWLOC_OBJ_L2CACHE)
+      && !hwloc_get_sysctlbyname(name, &size)) {
     int64_t cpus;
 
     hwloc_debug("found perflevel %u l2cachesize %ld\n", level, (long) size);
@@ -492,17 +495,18 @@ static void hwloc__darwin_look_perflevel_caches(struct hwloc_topology *topology,
       size_t s;
       snprintf(name, sizeof(name), "hw.perflevel%u.l2perflevels", level);
       if (!sysctlbyname(name, NULL, &s, NULL, 0))
-        if (!hwloc_hide_errors())
+        if (HWLOC_SHOW_ALL_ERRORS())
           fprintf(stderr, "hwloc/darwin: key %s succeeded size %lu, please report to hwloc developers.\n", name, s);
     }
 
     /* assume PUs are contigous for now. */
     hwloc__darwin_build_perflevel_cache_level(topology, cpuset, cpus, HWLOC_OBJ_L2CACHE, 2, size, linesize);
     gothybrid->l2++;
-}
+  }
 
   snprintf(name, sizeof(name), "hw.perflevel%u.l3cachesize", level);
-  if (!hwloc_get_sysctlbyname(name, &size)) {
+  if (hwloc_filter_check_keep_object_type(topology, HWLOC_OBJ_L3CACHE)
+      && !hwloc_get_sysctlbyname(name, &size)) {
     int64_t cpus;
 
     hwloc_debug("found perflevel %u l3cachesize %ld\n", level, (long) size);
@@ -522,7 +526,7 @@ static void hwloc__darwin_look_perflevel_caches(struct hwloc_topology *topology,
       size_t s;
       snprintf(name, sizeof(name), "hw.perflevel%u.l3perflevels", level);
       if (!sysctlbyname(name, NULL, &s, NULL, 0))
-        if (!hwloc_hide_errors())
+        if (HWLOC_SHOW_ALL_ERRORS())
           fprintf(stderr, "hwloc/darwin: key %s succeeded size %lu, please report to hwloc developers.\n", name, s);
     }
 
