@@ -1,5 +1,5 @@
 /*
- * Copyright © 2012-2022 Inria.  All rights reserved.
+ * Copyright © 2012-2023 Inria.  All rights reserved.
  * See COPYING in top-level directory.
  */
 
@@ -41,7 +41,7 @@ hwloc__nvml_get_peer_obj_by_pci(struct hwloc_topology *topology, hwloc_obj_t gpu
     /* we need PCI devices to be filtered-in */
     if (pfilter != HWLOC_TYPE_FILTER_KEEP_NONE) {
       static int warned = 0;
-      if (!warned && !hwloc_hide_errors())
+      if (!warned && HWLOC_SHOW_ALL_ERRORS())
         fprintf(stderr, "hwloc failed to find NVLink peer %04x:%02x:%02x\n",
                 peer_bdf.domain, peer_bdf.bus, peer_bdf.device);
       warned = 1;
@@ -89,7 +89,7 @@ hwloc__nvml_get_peer_obj_by_pci(struct hwloc_topology *topology, hwloc_obj_t gpu
   }
   default: {
     static int warned = 0;
-    if (!warned && !hwloc_hide_errors())
+    if (!warned && HWLOC_SHOW_ALL_ERRORS())
       fprintf(stderr, "hwloc failed to recognize NVLink peer %04x:%02x:%02x class %04x vendor %04x device %04x\n",
               peer_bdf.domain, peer_bdf.bus, peer_bdf.device,
               obj->attr->pcidev.class_id, obj->attr->pcidev.vendor_id, obj->attr->pcidev.device_id);
@@ -177,7 +177,7 @@ hwloc_nvml_discover(struct hwloc_backend *backend, struct hwloc_disc_status *dst
 
   ret = nvmlInit();
   if (NVML_SUCCESS != ret) {
-    if (!hwloc_hide_errors()) {
+    if (HWLOC_SHOW_ALL_ERRORS()) {
       const char *error = nvmlErrorString(ret);
       fprintf(stderr, "hwloc/nvml: Failed to initialize with nvmlInit(): %s\n", error);
     }
@@ -352,14 +352,21 @@ hwloc_nvml_discover(struct hwloc_backend *backend, struct hwloc_disc_status *dst
           continue;
 
         hwloc_debug("GPU #%u NVLink #%u has version %u\n", i, j, version);
-        /* version1 = 20GB/s, version2=25GB/s */
+        /* NVIDIA often shows bidirection bandwidths,
+         * or even the bidirectional bandwidth of all links agregated for a single GPU
+         * 160GB/s on P100 (4 links), 300 on V100 (6), 600 on A100 (12), 900 on H100 (18 links).
+         *
+         * The actual unidirectional bandwidth we want is 20GB/s for v1, and 25GB/s for v2+.
+         * v3 has twice bigger pairs than v2 but half the number of pairs per (sub-)link.
+         * v4 and v3 seem identical.
+         */
         if (version == 1) {
           bw = 20000; /* multiple links may connect same GPUs */
-        } else if (version == 2) {
+        } else if (version >= 2 && version <= 4) {
           bw = 25000; /* multiple links may connect same GPUs */
         } else {
           static int warned = 0;
-          if (!warned && !hwloc_hide_errors())
+          if (!warned && HWLOC_SHOW_ALL_ERRORS())
             fprintf(stderr, "Failed to recognize NVLink version %u\n", version);
           warned = 1;
           continue;
