@@ -1,6 +1,6 @@
 /*
  * Copyright © 2009 CNRS
- * Copyright © 2009-2022 Inria.  All rights reserved.
+ * Copyright © 2009-2023 Inria.  All rights reserved.
  * Copyright © 2009-2012, 2015, 2017 Université Bordeaux
  * Copyright © 2009-2011 Cisco Systems, Inc.  All rights reserved.
  * Copyright © 2020 Hewlett Packard Enterprise.  All rights reserved.
@@ -19,6 +19,9 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
 #ifdef HAVE_DIRENT_H
 #include <dirent.h>
 #endif
@@ -440,14 +443,10 @@ void usage(const char *name, FILE *where)
   fprintf (where, "See lstopo(1) for more details.\n");
 
   fprintf (where, "\nDefault output is "
-#ifdef LSTOPO_HAVE_GRAPHICS
-#ifdef HWLOC_WIN_SYS
-		  "graphical"
-#elif (defined LSTOPO_HAVE_X11)
-		  "graphical (X11) if DISPLAY is set, console otherwise"
-#else
-		  "console"
-#endif
+#if (defined LSTOPO_HAVE_GRAPHICS) && (defined HWLOC_WIN_SYS)
+		  "graphical window"
+#elif (defined LSTOPO_HAVE_GRAPHICS) && (defined LSTOPO_HAVE_X11)
+		  "graphical window (X11) if DISPLAY is set, console otherwise"
 #else
 		  "console"
 #endif
@@ -574,6 +573,12 @@ void usage(const char *name, FILE *where)
 
 void lstopo_show_interactive_help(void)
 {
+#ifdef HAVE_ISATTY
+  if (!isatty(STDOUT_FILENO))
+    /* don't send the interactive help when not a terminal */
+    return;
+#endif
+
   printf("\n");
   printf("Keyboard shortcuts:\n");
   printf(" Zooming, scrolling and closing:\n");
@@ -652,6 +657,7 @@ void lstopo_show_interactive_cli_options(const struct lstopo_output *loutput)
 
 enum output_format {
   LSTOPO_OUTPUT_DEFAULT,
+  LSTOPO_OUTPUT_WINDOW,
   LSTOPO_OUTPUT_CONSOLE,
   LSTOPO_OUTPUT_SYNTHETIC,
   LSTOPO_OUTPUT_ASCII,
@@ -673,6 +679,8 @@ parse_output_format(const char *name, char *callname __hwloc_attribute_unused)
 {
   if (!hwloc_strncasecmp(name, "default", 3))
     return LSTOPO_OUTPUT_DEFAULT;
+  else if (!hwloc_strncasecmp(name, "window", 3))
+    return LSTOPO_OUTPUT_WINDOW;
   else if (!hwloc_strncasecmp(name, "console", 3))
     return LSTOPO_OUTPUT_CONSOLE;
   else if (!strcasecmp(name, "synthetic"))
@@ -1425,7 +1433,8 @@ main (int argc, char *argv[])
         opt = 1;
       } else {
 	if (filename) {
-	  fprintf (stderr, "Unrecognized option: %s\n", argv[0]);
+	  fprintf (stderr, "Unrecognized option `%s', cannot be used as output filename (`%s' already given).\n",
+                   argv[0], filename);
 	  goto out_usagefailure;
 	} else
 	  filename = argv[0];
@@ -1475,10 +1484,21 @@ main (int argc, char *argv[])
   }
 
   switch (output_format) {
-  case LSTOPO_OUTPUT_DEFAULT:
+  case LSTOPO_OUTPUT_DEFAULT: {
 #ifdef LSTOPO_HAVE_GRAPHICS
 #if (defined LSTOPO_HAVE_X11)
-    if (getenv("DISPLAY")) {
+    int want_console = 0;
+#if (defined HAVE_ISATTY) && (defined HAVE_TCGETPGRP)
+    /* If stdout isn't a tty, we're likely redirecting stdout, use console mode.
+     * However, if launched from the window manager, we still want graphical mode.
+     * tcgetpgrp(STDIN) should fail with ENOTTY in this case.
+     * We don't specifically check for errno==ENOTTY: if tcgetpgrp() ever fails
+     * for another reason, don't assume we're redirected, keep the graphical mode too.
+     */
+    if (!isatty(STDOUT_FILENO) && tcgetpgrp(STDIN_FILENO) != -1)
+      want_console = 1;
+#endif
+    if (getenv("DISPLAY") && !want_console) {
       output_func = output_x11;
     } else
 #endif /* LSTOPO_HAVE_X11 */
@@ -1496,6 +1516,23 @@ main (int argc, char *argv[])
 #ifdef ANDROID
     setJNIEnv();
     output_func = output_android;
+#endif
+    }
+    break;
+
+  case LSTOPO_OUTPUT_WINDOW:
+#if (defined LSTOPO_HAVE_GRAPHICS) && (defined LSTOPO_HAVE_X11)
+    if (getenv("DISPLAY")) {
+      output_func = output_x11;
+    } else {
+      fprintf(stderr, "X11 graphical window output requires a DISPLAY environment variable.\n");
+      goto out;
+    }
+#elif (defined LSTOPO_HAVE_GRAPHICS) && (defined HWLOC_WIN_SYS)
+    output_func = output_windows;
+#else
+    fprintf(stderr, "Graphical window output not supported.\n");
+    goto out;
 #endif
     break;
 
